@@ -44,10 +44,6 @@ typedef long double LONG_DOUBLE;
 typedef double LONG_DOUBLE;
 #endif
 
-#if HAVE_VALUES_H
-# include <values.h>
-#endif
-
 /* The default number of input bytes per output line.  */
 #define DEFAULT_BYTES_PER_BLOCK 16
 
@@ -742,6 +738,11 @@ this system doesn't provide a %lu-byte integral type"), s_orig, size);
 	  break;
 	}
 
+#define ISPEC_TO_FORMAT(Spec, Min_format, Long_format, Max_format)	\
+  ((Spec) == LONG_LONG ? (Max_format)					\
+   : ((Spec) == LONG ? (Long_format)					\
+      : (Min_format)))							\
+
 #define FMT_BYTES_ALLOCATED 9
       fmt_string = xmalloc (FMT_BYTES_ALLOCATED);
 
@@ -751,32 +752,30 @@ this system doesn't provide a %lu-byte integral type"), s_orig, size);
 	{
 	case 'd':
 	  fmt = SIGNED_DECIMAL;
-	  sprintf (fmt_string, " %%%u%sd",
+	  sprintf (fmt_string, " %%%u%s",
 		   (field_width = bytes_to_signed_dec_digits[size]),
-		   (size_spec == LONG ? "l"
-		    : (size_spec == LONG_LONG ? "ll"
-		       : "")));
+		   ISPEC_TO_FORMAT (size_spec, "d", "ld", PRIdMAX));
 	  break;
 
 	case 'o':
 	  fmt = OCTAL;
-	  sprintf (fmt_string, " %%0%u%so",
+	  sprintf (fmt_string, " %%0%u%s",
 		   (field_width = bytes_to_oct_digits[size]),
-		   (size_spec == LONG ? "l" : ""));
+		   ISPEC_TO_FORMAT (size_spec, "o", "lo", PRIoMAX));
 	  break;
 
 	case 'u':
 	  fmt = UNSIGNED_DECIMAL;
-	  sprintf (fmt_string, " %%%u%su",
+	  sprintf (fmt_string, " %%%u%s",
 		   (field_width = bytes_to_unsigned_dec_digits[size]),
-		   (size_spec == LONG ? "l" : ""));
+		   ISPEC_TO_FORMAT (size_spec, "u", "lu", PRIuMAX));
 	  break;
 
 	case 'x':
 	  fmt = HEXADECIMAL;
-	  sprintf (fmt_string, " %%0%u%sx",
+	  sprintf (fmt_string, " %%0%u%s",
 		   (field_width = bytes_to_hex_digits[size]),
-		   (size_spec == LONG ? "l" : ""));
+		   ISPEC_TO_FORMAT (size_spec, "x", "lx", PRIxMAX));
 	  break;
 
 	default:
@@ -1094,7 +1093,7 @@ skip (uintmax_t n_skip)
 
 	  if (S_ISREG (file_stats.st_mode) && 0 <= file_stats.st_size)
 	    {
-	      if (file_stats.st_size <= n_skip)
+	      if ((uintmax_t) file_stats.st_size <= n_skip)
 		n_skip -= file_stats.st_size;
 	      else
 		{
@@ -1646,6 +1645,8 @@ main (int argc, char **argv)
   integral_type_size[sizeof (int)] = INT;
   integral_type_size[sizeof (long int)] = LONG;
 #if HAVE_UNSIGNED_LONG_LONG
+  /* If `long' and `long long' have the same size, it's fine
+     to overwrite the entry for `long' with this one.  */
   integral_type_size[sizeof (ulonglong_t)] = LONG_LONG;
 #endif
 
